using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Chart;
using System.Diagnostics;
using Syncfusion.Drawing;

namespace SFChartWindow
{
   public partial class SFChart : UserControl
   {
      public SFChart()
      {
         InitializeComponent();
         InitializeDictsAndLists();
         GetCellData();
         PrepareChart();
      }

      private ChartStripLine cursor = new ChartStripLine();
      public Dictionary<ChartSeries, LineParameterData> SeriesToLPD;
      private void InitializeDictsAndLists()
      {
         if (SeriesToLPD == null)
            SeriesToLPD = new Dictionary<ChartSeries, LineParameterData>(10);
         else
            SeriesToLPD.Clear();
      }

      private void PrepareChart()
      {
         FindChartLines();

         cursor.Enabled = true;
         cursor.Vertical = true;
         cursor.Text = "";
         cursor.Start = 0;
         cursor.End = 1;
         cursor.Width = 1;
         cursor.Interior = new BrushInfo(100, new BrushInfo(Color.Orange));

         this.hScrollBar1.Maximum = 150;
         this.hScrollBar1.Minimum = -20;

         this.contextMenuStrip1.ItemClicked += new ToolStripItemClickedEventHandler(contextMenuStrip1_ItemClicked);
         
         this.chartControl1.EnableXZooming = true;
         this.chartControl1.EnableYZooming = true;
         this.chartControl1.KeyZoom = true;
         this.chartControl1.ZoomCancel = (Keys)MouseButtons.Right;
         this.chartControl1.ZoomOutIncrement = 1.5;

         this.chartControl1.PrimaryXAxis.RangeType = ChartAxisRangeType.Set;
         this.chartControl1.PrimaryXAxis.Range.Min = 0.0;
         this.chartControl1.PrimaryXAxis.Range.Max = 200;
         this.chartControl1.PrimaryXAxis.Range.Interval = 50;
         this.chartControl1.PrimaryXAxis.StripLines.Add(cursor);
         this.chartControl1.PrimaryXAxis.DrawGrid = false;

         this.chartControl1.PrimaryYAxis.RangeType = ChartAxisRangeType.Set;
         this.chartControl1.PrimaryYAxis.Range.Min = 0.0;
         this.chartControl1.PrimaryYAxis.Range.Max = 50;
         this.chartControl1.PrimaryYAxis.Range.Interval = 10;
         this.chartControl1.VisibleRangeChanged += new EventHandler(chartControl1_VisibleRangeChanged);

         this.propertyGrid1.SelectedObject = this.chartControl1.Series[0];
      }

      void chartControl1_VisibleRangeChanged(object sender, EventArgs e)
      {
         double xPos = this.cursor.Start;
         if ((xPos >= chartControl1.PrimaryXAxis.VisibleRange.Min) && (xPos <= chartControl1.PrimaryXAxis.VisibleRange.Max))
            cursor.Enabled = true;
         else
            cursor.Enabled = false; 

      }

      LineParameterData lpd;
      LineParameterData lpd2;
      private void FindChartLines()
      {
         lpd = new LineParameterData();
         PrepareChartLine(lpd);
      }
     
      private void PrepareChartLine(LineParameterData lpd)
      {
         ChartSeries chartLine = this.chartControl1.Model.NewSeries(" Series 1",ChartSeriesType.Line);

         lpd.Series = chartLine;
         lpd.Series.Name = "TeSt";
         lpd.TestData = 19;
         chartLine.Style.Border.Width = 4;
         chartLine.Visible = true;
         chartLine.Style.Symbol.Shape = ChartSymbolShape.Circle;
         chartLine.Points.Add(12, 27);
         chartLine.Points.Add(14, 3);
         chartLine.Points.Add(34, 32);

         this.chartControl1.Series.Add(chartLine);
         SeriesToLPD.Add(chartLine, lpd);

         ChartSeries chartLine2 = this.chartControl1.Model.NewSeries(" Series 2", ChartSeriesType.StepLine);

         lpd2 = new LineParameterData();
         lpd2.Series = chartLine2;
         lpd2.Series.Name = "HuGo";

         chartLine2.Style.Symbol.Shape = ChartSymbolShape.Circle;
         chartLine2.Points.Add(24, 5);
         chartLine2.Points.Add(35, 25);
         chartLine2.Points.Add(102, 45);
         chartLine2.Points.Add(122, 36);

         this.chartControl1.Series.Add(chartLine2);
         SeriesToLPD.Add(chartLine2, lpd2);
      }

      private void InsertNewPointInLine(ChartSeries chartLine, ChartPoint chartPoint)
      {
         int pointIndex = 0;
         int points = chartLine.Points.Count;

         for (int i = 0; i < points; i++)
         {
            if (chartLine.Points[i].X < lpd.TestData)
               pointIndex++;
            else
               chartLine.Points.Insert(pointIndex, chartPoint);
         } 
      }

      private void GetCellData()
      {
      }

      private ChartPoint clickedCP;
      private int clickedPointIndex;
      private ChartSeries manipulatedSeries;
      void chartControl1_ChartRegionClick(object sender, Syncfusion.Windows.Forms.Chart.ChartRegionMouseEventArgs e)
      {
         clickedCP = this.chartControl1.ChartArea.GetValueByPoint(new Point(e.Point.X, e.Point.Y));
         Point pt = this.chartControl1.ChartArea.GetPointByValue(clickedCP);
         clickedPointIndex = e.Region.PointIndex;
         Debug.WriteLine(string.Format("Seriesindex: {0}", e.Region.SeriesIndex));
         if (e.Region.IsChartPoint)
         {
            manipulatedSeries = chartControl1.Series[e.Region.SeriesIndex];

            if (clickedCP.X < (manipulatedSeries.Points[clickedPointIndex].X + 2))
            {
               if (clickedCP.X > (manipulatedSeries.Points[clickedPointIndex].X - 2))
               {
                  Debug.WriteLine("Hit ChartPoint in X direction");
                  if (clickedCP.YValues[0] < (manipulatedSeries.Points[clickedPointIndex].YValues[0] + 2))
                  {
                     if (clickedCP.YValues[0] > (manipulatedSeries.Points[clickedPointIndex].YValues[0] - 2))
                     {
                        BuildContextOnPoints(pt);
                        Debug.WriteLine("Hit ChartPoint in Y direction");
                     }
                     else
                        BuildContextBetweenPoints(pt);
                  }
                  else
                     BuildContextBetweenPoints(pt);
               }

               else
                  BuildContextBetweenPoints(pt);
            }
            else
               BuildContextBetweenPoints(pt);

            if (SeriesToLPD.ContainsKey(manipulatedSeries))
            {
               Debug.WriteLine(string.Format("Seriesname: {0}",SeriesToLPD[manipulatedSeries].Series.Name));
            }
            else
               Debug.WriteLine("Series is not known!");
         }
         Debug.WriteLine(string.Format("Point clicked at: {0} / {1}", e.Point.X, e.Point.Y));
         Debug.WriteLine(string.Format("ChartPoint clicked: X={0}, Y={1}", clickedCP.X, clickedCP.YValues[0]));
         Debug.WriteLine("--------------------------------------------");
      }

      private void BuildContextOnPoints(Point pt)
      {
         this.contextMenuStrip1.Items.Clear();
         this.contextMenuStrip1.Items.Add("Move Point");
         this.contextMenuStrip1.Items[0].Tag = "move";
         this.contextMenuStrip1.Items.Add("Change Value");
         this.contextMenuStrip1.Items[1].Tag = "change";
         this.contextMenuStrip1.Show(this.chartControl1, pt.X, pt.Y);
      }

      private void BuildContextBetweenPoints(Point pt)
      {
         this.contextMenuStrip1.Items.Clear();
         this.contextMenuStrip1.Items.Add("Insert Point");
         this.contextMenuStrip1.Items[0].Tag = "insert";
         this.contextMenuStrip1.Items.Add("Change Value");
         this.contextMenuStrip1.Items[1].Tag = "change";
         this.contextMenuStrip1.Show(this.chartControl1, pt.X, pt.Y);
      }

      void contextMenuStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
      {
         if ((string)e.ClickedItem.Tag == "insert")
         {
            Debug.WriteLine("--> insert point");
            InsertNewPointInLine(manipulatedSeries, clickedCP);
         }

         if ((string)e.ClickedItem.Tag == "move")
         {
            Debug.WriteLine("--> move point");
            MovePointOnLine(manipulatedSeries, clickedPointIndex);
         }

         if ((string)e.ClickedItem.Tag == "change")
         {
            Debug.WriteLine("--> change value");
         }
      }

      private void MovePointOnLine(ChartSeries manipulatedSeries, int manipulatedLinePointIndex)
      {
         ChartPoint cp = null;
         cp = manipulatedSeries.Points[manipulatedLinePointIndex];
      }

      void chartControl1_ChartRegionMouseDown(object sender, Syncfusion.Windows.Forms.Chart.ChartRegionMouseEventArgs e)
      {
         
      }

      void hScrollBar1_Scroll(object sender, System.Windows.Forms.ScrollEventArgs e)
      {
         cursor.Start = e.NewValue;
         cursor.End = e.NewValue + cursor.Width;
         this.chartControl1.Refresh();
      }
   }
}